# -*- coding: utf-8 -*-
# Date: 19.10.2019 22:02
# Author: MaximRaduntsev

import ipaddress as ip
from typing import Optional, Any, Callable
import socket

is_ip_address: Callable[[Any], Optional[str]] = lambda ip_host: str(ip.ip_address(ip_host)) \
    if type(ip_host) in (str, int) else None
is_host_name: Callable[[Any], Optional[Any]] = lambda host_name: host_name \
    if socket.gethostbyname(host_name) else None


def host_ping(host):
    try:
        host = is_ip_address(host)
    except Exception as e:
        try:
            host = is_host_name(host)
            # print(f'{host} - {e}, воспринимаю как доменное имя')
        except Exception as e:
            print(f'{host} - {e}, Не верные параметры узла')
    return host



# -*- coding: utf-8 -*-
# Date: 01.01.2020
# Author: MaximRaduntsev

from PySide2.QtCore import Qt, QFile
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QTabWidget, QMainWindow, QTableWidget, QWidget, \
    QHBoxLayout, QLineEdit, QPushButton, QGroupBox, QHeaderView, \
    QAction, QMessageBox, QTableWidgetItem, QDialog, QFileDialog, QApplication, QVBoxLayout

import ui.connect_window_UIs as connect_window_UIs
import ui.add_user as add_member
import ui.remove_user as del_member
import ui.modify_user as modify_user

from pathlib import Path
from dynaconf import settings
from dynaconf.loaders import yaml_loader as loader
from ipaddress import ip_address

from ui import style


class Main(QMainWindow):
    """
    Класс окна программы
    """

    def __init__(self, database, server):
        super(Main, self).__init__()
        self.database = database
        self.server = server
        self.tabs = QTabWidget(self)
        self.setWindowTitle("Server Manager")
        self.setWindowIcon(QIcon('ui/icons/icon.ico'))
        self.setGeometry(450, 150, 1100, 750)
        # self.setFixedSize(self.size())

        self.tool_bar()
        self.tabs.blockSignals(False)
        self.tabs.currentChanged.connect(self.func_tab_refresh)
        self.setCentralWidget(self.tabs)

        self.history_user_table = QTableWidget(self)
        self.active_user_table = QTableWidget(self)

        self.tab1 = QWidget(self)
        self.tab2 = QWidget(self)

        self.init_ui()
        self.show()

    def init_ui(self):
        """Функция настройки отображения шапок таблиц"""
        header1 = [
            'ID Клиента',
            'Имя Клиента',
            'IP Адрес',
            'Порт',
            'Время подключения']
        tab_name1 = 'Список подключенных клиентов'
        self.base_tab(self.active_user_table, header1, self.tab1, tab_name1)
        self.member_search_entry.setDisabled(True)

        header2 = [
            'Имя Клиента',
            'Последний раз входил',
            'Сообщений отправлено',
            'Сообщений получено']
        tab_name2 = 'История клиентов'
        self.base_tab(self.history_user_table, header2, self.tab2, tab_name2)

    def layout_tab(self, table, tab, tab_name):
        """
        Функция настройки layout
        :param table:
        :param tab:
        :param tab_name:
        :return:
        """
        # self.member_search_text = QLabel("Найти пользователя")
        self.member_search_entry = QLineEdit()
        self.member_search_button = QPushButton("Найти")
        self.member_search_button.clicked.connect(self.search_users)
        self.member_search_button.setStyleSheet(style.search_button_style())

        self.member_main_layout = QHBoxLayout()
        self.member_left_layout = QHBoxLayout()
        self.member_right_layout = QVBoxLayout()
        self.right_top_layout = QVBoxLayout()

        self.member_right_group_box = QGroupBox("Поиск клиентов")
        self.right_top_layout.setContentsMargins(20, 40, 20, 100)
        self.member_right_group_box.setStyleSheet(style.search_box_style())

        self.bottom_group_box = QGroupBox()

        self.right_top_layout.addWidget(self.member_search_entry)
        self.right_top_layout.addWidget(self.member_search_button)
        self.member_right_group_box.setLayout(self.right_top_layout)

        self.member_left_layout.addWidget(table)

        self.member_right_layout.addWidget(self.member_right_group_box, 20)
        self.member_right_layout.addWidget(self.bottom_group_box, 80)

        self.member_main_layout.addLayout(self.member_left_layout, 70)
        self.member_main_layout.addLayout(self.member_right_layout, 30)

        tab.setLayout(self.member_main_layout)

        self.tabs.addTab(tab, tab_name)

    def base_tab(self, table, header, tab, tab_name):
        """
        Базовый tab
        :param table:
        :param header:
        :param tab:
        :param tab_name:
        :return:
        """
        table.setColumnCount(len(header))
        table.setHorizontalHeaderLabels(header)
        table.verticalHeader().hide()
        table.resizeColumnsToContents()
        table.resizeRowsToContents()
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.doubleClicked.connect(self.selected_user)
        self.layout_tab(table, tab, tab_name)

    def tool_bar(self):
        """
        Функция настройки и расположения tool bar
        :return:
        """
        self.tb = self.addToolBar('Tool Bar')
        self.tb.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)

        self.config_window = QAction(
            QIcon('ui/img/db.png'), 'Настройка БД', self)
        self.tb.addAction(self.config_window)
        self.config_window.triggered.connect(self.func_config_window)
        self.tb.addSeparator()

        self.add_member = QAction(
            QIcon('ui/icons/user_add.ico'),
            'Новый клиент',
            self)
        self.tb.addAction(self.add_member)
        self.add_member.triggered.connect(self.func_add_user)
        self.tb.addSeparator()

        self.del_member = QAction(
            QIcon('ui/icons/user_delete.ico'),
            'Удалить клиента',
            self)
        self.tb.addAction(self.del_member)
        self.del_member.triggered.connect(self.func_del_user)
        self.tb.addSeparator()

        self.tab_refresh = QAction(
            QIcon('ui/icons/brain.ico'), 'Обновить', self)
        self.tb.addAction(self.tab_refresh)
        self.tab_refresh.triggered.connect(self.func_tab_refresh)
        self.tb.addSeparator()

    def func_config_window(self):
        """
        Открытие настроек сервера
        :return:
        """
        self.config_open()

    def func_del_user(self):
        """
        Функция открытия окна удаления пользователя
        :return:
        """
        self.rem_window = del_member.DelUserDialog(self.database, self.server)

    def func_add_user(self):
        """
        Функция открытия окна добавления пользователя
        :return:
        """
        self.new_user = add_member.AddUser(self.database, self.server)

    def selected_user(self):
        """
        Функция открытия окна выбора пользователя
        :return:
        """
        list_user = []
        for i in range(0, 4):
            list_user.append(
                self.history_user_table.item(
                    self.history_user_table.currentRow(),
                    i).text())
        user_name = list_user[0]
        # user_id = list_user[0]
        self.modify_user = modify_user.ModifyUser(self.database, user_name)
        self.modify_user.show()

    def search_users(self):
        """
        Функция открытия окна выбора пользователя
        :return:
        """
        value = self.member_search_entry.text()
        if value == '':
            QMessageBox.information(
                self, 'Warning', 'Запрос не может быть пустым')

        else:
            self.member_search_entry.setText('')
            results = self.database.get_user_by_like(value)
            if not results:
                QMessageBox.warning(self, 'Warning', 'Нет такого клиента')
            else:
                for i in reversed(range(self.history_user_table.rowCount())):
                    self.history_user_table.removeRow(i)

                for row_data in results:
                    row_number = self.history_user_table.rowCount()
                    self.history_user_table.insertRow(row_number)
                    for column_number, data in enumerate(row_data):
                        self.history_user_table.setItem(
                            row_number, column_number, QTableWidgetItem(
                                self.table_widget_item(data)))

    @staticmethod
    def table_widget_item(data):
        """
        Функция настройки QTableWidgetItem
        :param data:
        :return:
        """
        item = QTableWidgetItem()
        item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
        item.setText(str(data))
        return item

    def config_open(self):
        """Открытие окна настроек сервера"""
        global config_window
        config_window = ConfigWindow(self)

    def display_users(self):
        """Функция таблицы пользователей во вкладке история клиентов """
        for i in reversed(range(self.history_user_table.rowCount())):
            self.history_user_table.removeRow(i)

        members = set(self.database.message_history())
        sorted(members, key=lambda item: item[3])
        for row_data in members:
            row_number = self.history_user_table.rowCount()
            self.history_user_table.insertRow(row_number)
            for column_number, data in enumerate(row_data):
                self.history_user_table.setItem(
                    row_number, column_number, QTableWidgetItem(
                        self.table_widget_item(data)))

    def func_tab_refresh(self):
        """Функция обновления таблицы пользователей во вкладке история клиентов """
        self.display_users()


class ConfigWindow(QDialog):
    """Класс окна настроек."""

    def __init__(self, parent):
        """Инициализация."""
        self.parent_gui = parent
        super().__init__()
        self.ui = connect_window_UIs.Ui_ConnectDialog()
        self.ui.setupUi(self)
        # self.ui = self.load_ui_widget("ui/connect_window.ui")
        self.init_ui()

    @staticmethod
    def load_ui_widget(ui_filename):
        """Функция загрузки ui"""
        ui_file = QFile(ui_filename)
        ui_file.open(QFile.ReadOnly)
        loader = QUiLoader()
        ui = loader.load(ui_file)
        ui_file.close()
        ui.show()
        return ui

    def init_ui(self):
        """Инициализация интерфейса."""
        self.setAttribute(Qt.WA_DeleteOnClose)
        db_def_name = Path(settings.get('DATABASES.SERVER.NAME'))

        def open_file_dialog():
            """Функция обработчик открытия окна выбора папки."""
            global dialog
            dialog = QFileDialog(config_window)
            path_d = dialog.getExistingDirectory()
            if path_d:
                path_d = str(Path(path_d))
                self.ui.db_path_lbl.clear()
                self.ui.db_path_lne.insert(path_d)

        self.ui.db_path_btn.clicked.connect(open_file_dialog)
        self.ui.db_path_lne.insert(str(db_def_name.parent.absolute()))
        self.ui.db_name_lne.insert(str(db_def_name.name))
        self.ui.ip_lne.insert(
            str(ip_address(settings.get('DEFAULT_SERVER_IP'))))
        self.ui.port_lne.insert(str(settings.get('DEFAULT_PORT')))
        self.ui.btn_box.accepted.connect(self.save_server_config)
        self.show()

    def save_server_config(self):
        """Функция сохранения настроек сервера."""
        for_save = dict()
        msg_box = QMessageBox()
        db_path = Path(self.ui.db_path_lne.text())
        db_name = Path(self.ui.db_name_lne.text())
        db = db_path.joinpath(db_name)
        try:
            db = db.relative_to(Path.cwd())
        except Exception:
            pass
        try:
            port = self.parent_gui.port = int(self.ui.port_lne.text())
        except Exception as e:
            return msg_box.warning(config_window, 'Ошибка', str(e))
        # set settings
        settings.set('DEFAULT_SERVER_IP', self.ui.ip_lne.text())
        settings.set('DEFAULT_PORT', port)
        settings.DATABASES.SERVER.NAME = str(db)
        for_save['DEFAULT'] = set_dict = settings.as_dict()
        for_save['DEFAULT']['DATABASES'] = set_dict['DATABASES']

        loader.write(Path('config/server_settings.yaml'), for_save)
        msg_box.information(
            config_window,
            'OK',
            'Настройки успешно сохранены!')


if __name__ == '__main__':
    app = QApplication()
    ex = Main(None, None)
    app.exec_()



# Date: 22/09/2019 19:22
# Author: MaximRaduntsev

import sys
import logging.handlers
import os.path
from dynaconf import settings

# Подготовка имени файла для логирования
BASE_LOG_DIR_PATH = os.path.abspath(os.path.join(__file__, '../'))
LOG_PATH = os.path.join(BASE_LOG_DIR_PATH, 'log_storage/server.log')

# Создаем объект форматирования: <дата-время> <уровень важности> <имя
# модуля> <сообщение>
formatter = logging.Formatter(
    '%(asctime)s - %(levelname)s - %(module)s: %(message)s',
    '%Y-%m-%d %H:%M:%S')

# Создать обработчик, который выводит сообщения в поток stderr
stream = logging.StreamHandler(sys.stderr)
stream.setFormatter(formatter)
stream.setLevel(logging.ERROR)

# Создаем файловый обработчик логирования:
log_file = logging.handlers.TimedRotatingFileHandler(
    LOG_PATH, encoding=settings.BASE_ENCODING, interval=1, when='D')
log_file.setFormatter(formatter)

# Создать объект-логгер регистратор
logger = logging.getLogger('server')
logger.addHandler(log_file)
logger.addHandler(stream)
logger.setLevel(settings.LOGGING_LEVEL)

if __name__ == '__main__':
    logger.info('Тестовый запуск логирования')
    logger.critical('Тестовый запуск логирования')
    logger.error('Тестовый запуск логирования')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date: 11.12.2019 21:19
# @Author: MaximRaduntsev

"""Утилиты для работы с протоколом."""

import time
from dynaconf import settings


class Message:
    """
    Класс сообщения. Отдает свои атрибуты как ключи из данных и без генерации
    ошибок
    """

    def __init__(self, data=None, **kwargs):
        if data:
            if isinstance(data, dict):
                data.update(kwargs)
            kwargs = data
        self.__raw = kwargs
        date_format = kwargs.pop('date_format', '%Y-%m-%d %H:%M:%S')
        self.__raw[settings.TIME] = time.strftime(date_format)

    def __str__(self):
        return str(self.__raw)

    @property
    def to_dict(self):
        """Словарь с данными"""
        return self.__raw

    @property
    def action(self):
        """Действие"""
        return self.__raw.get(settings.ACTION)

    @property
    def response(self):
        """Код ответа"""
        return self.__raw.get(settings.RESPONSE)

    @property
    def user(self):
        """Информация о клиенте"""
        try:
            return self.__raw[settings.USER].get(settings.ACCOUNT_NAME)
        except ValueError:
            return None
        except Exception:
            data = (settings.USER, settings.ACCOUNT_NAME,
                    settings.SENDER, settings.DESTINATION)
            for d in data:
                name = self.__raw.get(d)
                if name:
                    # print('name = ', name)
                    return name

    @property
    def public_key(self):
        """Ключ"""
        return self.__raw[settings.USER].get(settings.PUBLIC_KEY)

    @property
    def destination(self):
        """Получатель"""
        return self.__raw.get(settings.DESTINATION)

    @property
    def sender(self):
        """Отправитель"""
        return self.__raw.get(settings.SENDER)

    @property
    def error(self):
        return self.__raw.get(settings.ERROR)

    @property
    def text(self):
        """Переданный текст"""
        return self.__raw.get(settings.MESSAGE_TEXT)

    @classmethod
    def error_resp(cls, text, **kwargs):
        """
        Ошибка запроса пользователя
        """
        return cls(response=settings.BAD_REQUEST, error=text, **kwargs)

    @classmethod
    def error_request(cls, text, **kwargs):
        """
        Ошибка.
        """
        return cls(action=settings.ERROR, msg=text, **kwargs)

    @classmethod
    def success(cls, response=settings.OK, **kwargs):
        """
        Сообщение об успехе.
        """
        return cls(response=response, **kwargs)

    @classmethod
    def accepted(cls, response=settings.ACCEPTED, **kwargs):
        """
        Сообщение о разрешении действия.
        """
        return cls(response=response, **kwargs)

    @classmethod
    def forbidden(cls, response=settings.BAD_REQUEST, **kwargs):
        """
        Сообщение о запрете действия
        """
        return cls(response=response, **kwargs)

    @classmethod
    def presence(cls, logger, account_name, status, pubkey, **kwargs):
        """
        Презентационное сообщение
        """
        if status:
            user_name = {settings.ACCOUNT_NAME: account_name}
            # settings.STATUS: status}
            message = {settings.ACTION: settings.PRESENCE,
                       settings.STATUS: status,
                       settings.USER: user_name}
        else:
            user_name = {settings.ACCOUNT_NAME: account_name,
                         settings.PUBLIC_KEY: pubkey
                         }
            message = {settings.ACTION: settings.PRESENCE,
                       settings.USER: user_name or settings.ACCOUNT_NAME,
                       }
        info_msg = f'Сформировано {message} сообщение для ' \
                   f'пользователя {account_name}'
        logger.info(info_msg)
        return cls(message, **kwargs)

    @classmethod
    def exit_message(cls, logger, user=None, **kwargs):
        """
        Завершение сеанса
        """
        info_msg = 'Завершение работы по команде пользователя.'
        logger.info(info_msg)
        return cls(
            action=settings.QUIT,
            account_name=user or settings.ACCOUNT_NAME,
            **kwargs)


if __name__ == "__main__":
    print(settings.RESPONSE_OK)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'connect_window.ui'
##
## Created by: Qt User Interface Compiler version 5.14.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QMetaObject, QObject, QPoint,
    QRect, QSize, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QFont,
    QFontDatabase, QIcon, QLinearGradient, QPalette, QPainter, QPixmap,
    QRadialGradient)
from PySide2.QtWidgets import *

class Ui_ConnectDialog(object):
    def setupUi(self, ConnectDialog):
        if ConnectDialog.objectName():
            ConnectDialog.setObjectName(u"ConnectDialog")
        ConnectDialog.resize(400, 273)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ConnectDialog.sizePolicy().hasHeightForWidth())
        ConnectDialog.setSizePolicy(sizePolicy)
        ConnectDialog.setMinimumSize(QSize(400, 200))
        ConnectDialog.setMaximumSize(QSize(500, 300))
        icon = QIcon()
        icon.addFile(u"ui/icons/db.png", QSize(), QIcon.Normal, QIcon.Off)
        ConnectDialog.setWindowIcon(icon)
        ConnectDialog.setSizeGripEnabled(True)
        ConnectDialog.setModal(True)
        self.gridLayout = QGridLayout(ConnectDialog)
        self.gridLayout.setObjectName(u"gridLayout")
        self.hlLayout_1 = QHBoxLayout()
        self.hlLayout_1.setObjectName(u"hlLayout_1")
        self.db_name_lbl = QLabel(ConnectDialog)
        self.db_name_lbl.setObjectName(u"db_name_lbl")
        self.db_name_lbl.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignVCenter)

        self.hlLayout_1.addWidget(self.db_name_lbl)

        self.db_name_lne = QLineEdit(ConnectDialog)
        self.db_name_lne.setObjectName(u"db_name_lne")

        self.hlLayout_1.addWidget(self.db_name_lne)

        self.hlLayout_1.setStretch(0, 2)
        self.hlLayout_1.setStretch(1, 3)

        self.gridLayout.addLayout(self.hlLayout_1, 1, 0, 1, 1)

        self.vlLayout_1 = QVBoxLayout()
        self.vlLayout_1.setSpacing(0)
        self.vlLayout_1.setObjectName(u"vlLayout_1")
        self.db_path_lbl = QLabel(ConnectDialog)
        self.db_path_lbl.setObjectName(u"db_path_lbl")

        self.vlLayout_1.addWidget(self.db_path_lbl)

        self.hlLayout_2 = QHBoxLayout()
        self.hlLayout_2.setObjectName(u"hlLayout_2")
        self.db_path_lne = QLineEdit(ConnectDialog)
        self.db_path_lne.setObjectName(u"db_path_lne")

        self.hlLayout_2.addWidget(self.db_path_lne)

        self.db_path_btn = QPushButton(ConnectDialog)
        self.db_path_btn.setObjectName(u"db_path_btn")

        self.hlLayout_2.addWidget(self.db_path_btn)

        self.hlLayout_2.setStretch(0, 3)
        self.hlLayout_2.setStretch(1, 1)

        self.vlLayout_1.addLayout(self.hlLayout_2)


        self.gridLayout.addLayout(self.vlLayout_1, 0, 0, 1, 1)

        self.vlLayout_2 = QVBoxLayout()
        self.vlLayout_2.setSpacing(0)
        self.vlLayout_2.setObjectName(u"vlLayout_2")
        self.hlLayout_3 = QHBoxLayout()
        self.hlLayout_3.setSpacing(6)
        self.hlLayout_3.setObjectName(u"hlLayout_3")
        self.ip_lbl = QLabel(ConnectDialog)
        self.ip_lbl.setObjectName(u"ip_lbl")

        self.hlLayout_3.addWidget(self.ip_lbl)

        self.ip_lne = QLineEdit(ConnectDialog)
        self.ip_lne.setObjectName(u"ip_lne")
        self.ip_lne.setInputMask(u"000.000.000.000;_")

        self.hlLayout_3.addWidget(self.ip_lne)

        self.hlLayout_3.setStretch(0, 2)
        self.hlLayout_3.setStretch(1, 3)

        self.vlLayout_2.addLayout(self.hlLayout_3)

        self.info_lbl = QLabel(ConnectDialog)
        self.info_lbl.setObjectName(u"info_lbl")

        self.vlLayout_2.addWidget(self.info_lbl)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.vlLayout_2.addItem(self.verticalSpacer)


        self.gridLayout.addLayout(self.vlLayout_2, 3, 0, 1, 1)

        self.btn_box = QDialogButtonBox(ConnectDialog)
        self.btn_box.setObjectName(u"btn_box")
        self.btn_box.setOrientation(Qt.Horizontal)
        self.btn_box.setStandardButtons(QDialogButtonBox.Close|QDialogButtonBox.Save)
        self.btn_box.setCenterButtons(False)

        self.gridLayout.addWidget(self.btn_box, 4, 0, 1, 1)

        self.hlLayout_4 = QHBoxLayout()
        self.hlLayout_4.setObjectName(u"hlLayout_4")
        self.port_lbl = QLabel(ConnectDialog)
        self.port_lbl.setObjectName(u"port_lbl")

        self.hlLayout_4.addWidget(self.port_lbl)

        self.port_lne = QLineEdit(ConnectDialog)
        self.port_lne.setObjectName(u"port_lne")

        self.hlLayout_4.addWidget(self.port_lne)

        self.hlLayout_4.setStretch(0, 2)
        self.hlLayout_4.setStretch(1, 3)

        self.gridLayout.addLayout(self.hlLayout_4, 2, 0, 1, 1)

        QWidget.setTabOrder(self.db_path_lne, self.db_path_btn)
        QWidget.setTabOrder(self.db_path_btn, self.db_name_lne)
        QWidget.setTabOrder(self.db_name_lne, self.ip_lne)

        self.retranslateUi(ConnectDialog)
        self.btn_box.accepted.connect(ConnectDialog.accept)
        self.btn_box.rejected.connect(ConnectDialog.reject)

        QMetaObject.connectSlotsByName(ConnectDialog)
    # setupUi

    def retranslateUi(self, ConnectDialog):
        ConnectDialog.setWindowTitle(QCoreApplication.translate("ConnectDialog", u"\u041d\u0430\u0441\u0442\u0440\u043e\u0439\u043a\u0438 \u0441\u0435\u0440\u0432\u0435\u0440\u0430", None))
        self.db_name_lbl.setText(QCoreApplication.translate("ConnectDialog", u"\u0418\u043c\u044f \u0444\u0430\u0439\u043b\u0430 \u0431\u0430\u0437\u044b \u0434\u0430\u043d\u043d\u044b\u0445:", None))
        self.db_name_lne.setPlaceholderText(QCoreApplication.translate("ConnectDialog", u"\u0418\u043c\u044f \u0431\u0430\u0437\u044b \u0434\u0430\u043d\u043d\u044b\u0445", None))
        self.db_path_lbl.setText(QCoreApplication.translate("ConnectDialog", u"\u041f\u0443\u0442\u044c \u0434\u043e \u0444\u0430\u0439\u043b\u0430 \u0431\u0430\u0437\u044b \u0434\u0430\u043d\u043d\u044b\u0445:", None))
        self.db_path_lne.setPlaceholderText(QCoreApplication.translate("ConnectDialog", u"\u041f\u0443\u0442\u044c \u043d\u0430 \u0434\u0438\u0441\u043a\u0435 \u043a \u0431\u0430\u0437\u0435 \u0434\u0430\u043d\u043d\u044b\u0445", None))
        self.db_path_btn.setText(QCoreApplication.translate("ConnectDialog", u"\u041e\u0431\u0437\u043e\u0440...", None))
        self.ip_lbl.setText(QCoreApplication.translate("ConnectDialog", u"\u0421 \u043a\u0430\u043a\u043e\u0433\u043e IP \u043f\u0440\u0438\u043d\u0438\u043c\u0430\u0435\u043c \u0441\u043e\u0435\u0434\u0438\u043d\u0435\u043d\u0438\u044f", None))
        self.info_lbl.setText(QCoreApplication.translate("ConnectDialog", u" \u043e\u0441\u0442\u0430\u0432\u044c\u0442\u0435 \u044d\u0442\u043e \u043f\u043e\u043b\u0435 \u043f\u0443\u0441\u0442\u044b\u043c, \u0447\u0442\u043e\u0431\u044b\n"
" \u043f\u0440\u0438\u043d\u0438\u043c\u0430\u0442\u044c \u0441\u043e\u0435\u0434\u0438\u043d\u0435\u043d\u0438\u044f \u0441 \u043b\u044e\u0431\u044b\u0445 \u0430\u0434\u0440\u0435\u0441\u043e\u0432.", None))
        self.port_lbl.setText(QCoreApplication.translate("ConnectDialog", u"\u041d\u043e\u043c\u0435\u0440 \u043f\u043e\u0440\u0442\u0430 \u0434\u043b\u044f \u0441\u043e\u0435\u0434\u0438\u043d\u0435\u043d\u0438\u0439:", None))
        self.port_lne.setPlaceholderText(QCoreApplication.translate("ConnectDialog", u"\u0412\u0432\u0435\u0434\u0438\u0442\u0435 \u043f\u043e\u0440\u0442", None))
    # retranslateUi




# Date: 22/09/2019 05:40
# Author: MaximRaduntsev

from dynaconf import settings


class MaxLongUsernameError(Exception):
    """
    Поле длинее, чем нужно
    """

    def __init__(self, name):
        """
        Ошибка длинного имени
        """
        self.name = name

    def __str__(self):
        """
        :return: str (Сообщение об ошибке)
        """
        return 'Name {} is too long: {} symbols, maximum allowed is {}'.format(
            self.name, len(self.name), settings.MAX_USERNAME_LENGTH)


class UnknownServerError(Exception):
    """
    Исключение - ошибка сервера
    """

    def __init__(self, text):
        self.text = text

    def __str__(self):
        return 'Unknown type of message: {}'.format(self.text)


class ResponseCodeError(Exception):
    """
    Исключение - ответ отсутствует среди списка ответов
    """

    def __init__(self, code):
        self.code = code

    def __str__(self):
        return 'Wrong response code: {}'.format(self.code)


class WrongActionError(Exception):
    """
    Неверное действие
    """

    def __init__(self, action):
        self.action = action

    def __str__(self):
        return 'Wrong action: {}'.format(self.action)


class MandatoryKeyError(Exception):
    def __init__(self, key):
        self.key = key

    def __str__(self):
        return 'Не хватает обязательного атрибута {}'.format(self.key)



# -*- coding: utf-8 -*-
# Date: 11.02.2020
# Author: MaximRaduntsev

from PySide2.QtCore import Qt
from PySide2.QtGui import QPixmap, QIcon
from PySide2.QtWidgets import QWidget, QPushButton, QLabel, QLineEdit, \
    QVBoxLayout, QFormLayout, QFrame, QMessageBox

import ui.style as style


class ModifyUser(QWidget):
    """
    Класс редактора пользователя
    """
    def __init__(self, database, user_name, password=None):
        super().__init__()
        self.database = database
        self.user_name = user_name
        self.password = password
        self.bottom_frame = QFrame()
        self.top_frame = QFrame()
        self.bottom_layout = QFormLayout()
        self.top_layout = QVBoxLayout()
        self.main_layout = QVBoxLayout()
        self.update_btn = QPushButton('Обновить')
        self.password_l_edit = QLineEdit()
        self.name_l_edit = QLineEdit()
        self.text_label = QLabel('Редактировать профиль')
        self.img = QPixmap('ui/img/members.png')
        self.user_img = QLabel()
        self.setWindowTitle('Редактор профиля')
        self.setWindowIcon(QIcon('ui/icons/icon.ico'))
        self.setGeometry(450, 150, 350, 600)
        self.setFixedSize(self.size())
        self.init_ui()
        self.show()

    def init_ui(self):
        """Функция инициализации окна"""
        self.user_details()
        self.widgets()
        self.layouts()

    def user_details(self):
        query = self.database.get_user_by_name(self.user_name)
        self.user_name = query.name
        # self.password = query.name

    def widgets(self):
        self.user_img.setPixmap(self.img)
        self.user_img.setAlignment(Qt.AlignCenter)
        self.text_label.setAlignment(Qt.AlignCenter)

        self.name_l_edit.setText(self.user_name)
        self.password_l_edit.setText(self.password)
        self.update_btn.clicked.connect(self.update_user)

    def layouts(self):
        self.top_frame.setStyleSheet(style.user_top_frame())
        self.bottom_frame.setStyleSheet(style.user_bottom_frame())

        self.top_layout.addWidget(self.text_label)
        self.top_layout.addWidget(self.user_img)
        self.top_frame.setLayout(self.top_layout)

        self.bottom_layout.addRow(QLabel('Имя: '), self.name_l_edit)
        self.bottom_layout.addRow(QLabel('Пароль: '), self.password_l_edit)
        # пока блокирую
        self.password_l_edit.setDisabled(True)
        self.bottom_layout.addRow(QLabel(''), self.update_btn)
        self.bottom_frame.setLayout(self.bottom_layout)

        self.main_layout.addWidget(self.top_frame)
        self.main_layout.addWidget(self.bottom_frame)
        self.setLayout(self.main_layout)

    def update_user(self):
        name = self.name_l_edit.text()
        password = self.password_l_edit.text()

        if name:
            try:
                self.database.user_modify(self.user_name, name)
                QMessageBox.information(
                    self, 'Info', 'Имя пользователя обновлено!')
            except BaseException:
                QMessageBox.warning(
                    self, 'Warning', 'Имя пользователя не обновлено!')
        else:
            QMessageBox.information(self, 'Info', 'Поле не может быть пустым!')



import sqlalchemy
import sqlalchemy.orm
from sqlalchemy.orm import Session, Query

from server_db.db import db_folder
from server_db.data.sqlalchemybase import SqlAlchemyBase

__engine = None
__factory = None


def global_init(db_name: str):
    global __engine, __factory

    if __factory:
        return

    conn_str = 'sqlite:///' + db_folder.get_full_path(db_name)
    __engine = sqlalchemy.create_engine(conn_str, echo=False, pool_recycle=7200,
                                        connect_args={'check_same_thread': False})
    __factory = sqlalchemy.orm.sessionmaker(bind=__engine, query_cls=Query)


def create_tables():
    if not __engine:
        raise Exception("You have not called global_init()")

    # noinspection PyUnresolvedReferences
    SqlAlchemyBase.metadata.create_all(__engine)


def create_session() -> sqlalchemy.orm.Session:
    if not __factory:
        raise Exception("You have not called global_init()")

    session: Session = __factory()
    session.expire_on_commit = False
    return session


def drop_tables():
    SqlAlchemyBase.metadata.drop_all(__engine)



def search_box_style():
    return """
    QGroupBox{
    background-color:#9bc9ff;
    font:15pt Times Bold;
    color:white;
    border:2px solid gray;
    border-radius:15px;
    }
    """


def search_button_style():
    return """
    QPushButton{
    background-color: #fcc324;
    border-style:outset;
    border-width:2px;
    border-radius:10px;
    border-color:beige;
    font:12px;
    padding:6px;
    min-width:6em;
    }
    """


def user_top_frame():
    return """
           QFrame{
           font:20pt Times Bold;
           background-color:white;
           }
           """


def user_bottom_frame():
    return """
        QFrame{
        font:15pt Times Bold;
        background-color:#fcc324
        }
        """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from jim.errors import MaxLongUsernameError
from common.host_ping import host_ping
from log.client_log_config import logger
from dynaconf import settings


class AddressField:
    """
    Дескриптор для сокета
    """

    def __init__(self):
        super().__init__()
        self._addr = None
        self._port = None

    def __set__(self, instance, value):
        addr, port = value

        if not host_ping(addr):
            logger.critical(f'{addr}, Не верные параметры узла')
            raise ValueError(f'{addr}, Не верные параметры узла')

        if isinstance(port, int) and 1023 < port < 65537:
            self._port = port
        else:
            logger.critical('Порт должен быть от 1024 до 65536')
            raise ValueError('Порт должен быть от 1024 до 65536')

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        # owner - <class '__main__.client'>
        # name - __name
        self.name = name


class ModeField:
    """
    Дескриптор выбора режима
    """

    def __init__(self, mode='r'):
        super().__init__()
        self._mode = mode

    def __set__(self, instance, value):
        # value - выбранный режим работы клиента
        # Проверим допустим ли выбранный режим работы клиента
        if value not in ('r', 'w'):
            logger.critical(f'Указан недопустимый режим работы {value}, '
                            f'допустимые режимы: r - чтение, w - запись')
            exit(1)

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        # owner - <class '__main__.Server'>
        # name - mode
        self.name = name

    def __get__(self, inst, inst_type=None):
        return self._mode


class LoginField:
    """
    Дескриптор имени клиента
    """

    def __set__(self, instance, value):

        if len(value) > settings.MAX_USERNAME_LENGTH:
            logger.critical(
                f'Имя больше {settings.MAX_USERNAME_LENGTH} символов')
            raise MaxLongUsernameError(value)

        if not isinstance(value, str):
            logger.critical(f'Имя {value} не строка')
            raise TypeError

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        # owner - <class '__main__.client'>
        # name - __name
        self.name = name


class MaxLengthField:
    """Дескриптор ограничивающий размер поля"""

    def __init__(self, name, max_length):
        """
        :param name: имя поля
        :param max_length: максимальная длина
        """
        self.max_length = max_length
        self.name = '_' + name

    def __set__(self, instance, value):
        # если длина поля больше максимального значения
        if len(value) > self.max_length:
            # вызываем ошибку
            raise MaxLongUsernameError(self.name)
        # иначе записываем данные в поле
        setattr(instance, self.name, value)

    def __get__(self, instance, owner):
        # получаем данные поля
        return getattr(instance, self.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date: 09.12.2019 11:14
# @Author: MaximRaduntsev

import datetime
from sqlalchemy import Column, Integer, String, ForeignKey, DateTime, func, Text
from server_db.data.sqlalchemybase import SqlAlchemyBase


class User(SqlAlchemyBase):
    """Класс - отображение таблицы всех пользователей."""
    __tablename__ = 'user'
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, unique=True, sqlite_on_conflict_unique='IGNORE')
    last_login = Column(String)
    pass_hash = Column(String)
    pubkey = Column(Text)

    def __init__(self, username, pass_hash):
        self.id = None
        self.name = username
        self.last_login = datetime.datetime.now()
        self.pass_hash = pass_hash
        self.pubkey = None


# Экземпляр этого класса = запись в таблице ActiveUsers
class ActiveUser(SqlAlchemyBase):
    """Класс - отображение таблицы активных пользователей."""
    __tablename__ = 'active_user'
    id = Column(Integer, primary_key=True, autoincrement=True)
    user = Column(ForeignKey('user.id'), unique=True, sqlite_on_conflict_unique='IGNORE')
    ip_address = Column(String)
    port = Column(Integer)
    login_time = Column(DateTime)

    def __init__(self, user_id, ip_address, port, login_time):
        self.id = None
        self.user = user_id
        self.ip_address = ip_address
        self.port = port
        self.login_time = login_time


# Экземпляр этого класса = запись в таблице LoginHistory
class LoginHistory(SqlAlchemyBase):
    """Класс - отображение таблицы истории входов."""
    __tablename__ = 'login_history'
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(ForeignKey('user.id', ondelete='CASCADE'))
    date_time = Column(DateTime, default=func.now(), onupdate=func.utc_timestamp())
    ip = Column(String)
    port = Column(Integer)

    def __init__(self, name, date, ip, port):
        self.id = None
        self.name = name
        self.date_time = date
        self.ip = ip
        self.port = port


class Contact(SqlAlchemyBase):
    """Класс - отображение таблицы контактов пользователей."""
    __tablename__ = 'contact'
    id = Column(Integer, primary_key=True)
    user = Column(ForeignKey('user.id'))
    contact = Column(ForeignKey('user.id'))

    def __init__(self, user, contact):
        self.id = None
        self.user = user
        self.contact = contact


class UserHistory(SqlAlchemyBase):
    """Класс - отображение таблицы истории действий."""
    __tablename__ = 'history'
    id = Column(Integer, primary_key=True)
    user = Column(ForeignKey(User.id))
    sent = Column(Integer)
    accepted = Column(Integer)

    def __init__(self, user):
        self.id = None
        self.user = user
        self.sent = 0
        self.accepted = 0



# -*- coding: utf-8 -*-
# Date: 06.02.2020
# Author: MaximRaduntsev
import os
from dynaconf import settings
from config.settings import __file__ as config1_file

f1 = os.path.join(os.path.dirname(config1_file), 'settings.py')
os.environ['SETTINGS_MODULE_FOR_DYNACONF'] = f1

if __name__ == "__main__":
    print(settings.RESPONSE_OK)



# -*- coding: utf-8 -*-
# Date: 09.02.2020
# Author: MaximRaduntsev

from PySide2.QtCore import Qt
from PySide2.QtGui import QPixmap, QIcon
from PySide2.QtWidgets import QWidget, QPushButton, QLabel, QLineEdit, \
    QVBoxLayout, QFormLayout, QFrame, QMessageBox
import hashlib
import binascii

from ui import style


class AddUser(QWidget):
    """
    Класс окна добавления пользователя
    """

    def __init__(self, database, server):
        super().__init__()
        self.database = database
        self.server = server

        self.password_l_edit = QLineEdit()
        self.label_conf = QLineEdit()
        self.bottom_frame = QFrame()
        self.top_frame = QFrame()
        self.bottom_layout = QFormLayout()
        self.top_layout = QVBoxLayout()
        self.main_layout = QVBoxLayout()
        self.submit_btn = QPushButton("Добавить")
        self.add_user_img = QLabel()
        self.name_l_edit = QLineEdit()
        self.text_label = QLabel("Добавьте пользователя")
        self.img = QPixmap('ui/img/add_user.png')
        self.setWindowTitle("Добавить клиента")
        self.setWindowIcon(QIcon('ui/icons/user_add.ico'))
        self.setGeometry(450, 150, 350, 550)
        self.setFixedSize(self.size())
        self.messages = QMessageBox()
        self.init_ui()
        self.show()

    def init_ui(self):
        """Функция инициализации окна"""
        self.widgets()
        self.layouts()

    def widgets(self):
        """Функция настройки виджета"""
        self.add_user_img.setPixmap(self.img)
        self.add_user_img.setAlignment(Qt.AlignCenter)
        self.text_label.setAlignment(Qt.AlignCenter)
        self.name_l_edit.setPlaceholderText('Введите имя пользователя')
        self.password_l_edit.setPlaceholderText('Пароль')
        self.label_conf.setPlaceholderText('Подтверждение')
        self.submit_btn.clicked.connect(self.save_data)

    def layouts(self):
        """Функция настройки layouts"""
        self.top_frame.setStyleSheet(style.user_top_frame())
        self.bottom_frame.setStyleSheet(style.user_bottom_frame())

        self.top_layout.addWidget(self.text_label)
        self.top_layout.addWidget(self.add_user_img)
        self.top_frame.setLayout(self.top_layout)
        self.bottom_layout.addRow(QLabel('Логин: '), self.name_l_edit)
        self.bottom_layout.addRow(QLabel('Пароль: '), self.password_l_edit)
        self.password_l_edit.setEchoMode(QLineEdit.Password)
        self.bottom_layout.addRow(QLabel('Подтверждение: '), self.label_conf)
        self.label_conf.setEchoMode(QLineEdit.Password)
        # self.password_l_edit.setDisabled(True)

        self.bottom_layout.addRow(QLabel(''), self.submit_btn)
        self.bottom_frame.setLayout(self.bottom_layout)

        self.main_layout.addWidget(self.top_frame)
        self.main_layout.addWidget(self.bottom_frame)

        self.setLayout(self.main_layout)

    def save_data(self):
        """
        Функция проверки правильности ввода и сохранения в базу
        нового пользователя.
        """
        if not self.name_l_edit.text():
            self.messages.critical(
                self, 'Ошибка', 'Не указано имя пользователя.')
            return
        elif self.password_l_edit.text() != self.label_conf.text():
            self.messages.critical(
                self, 'Ошибка', 'Введённые пароли не совпадают.')
            return
        elif self.database.check_user(self.name_l_edit.text()):
            self.messages.critical(
                self, 'Ошибка', 'Пользователь уже существует.')
            return
        else:
            # Генерируем хэш пароля, в качестве соли будем использовать логин в
            # нижнем регистре.
            passwd_bytes = self.password_l_edit.text().encode('utf-8')
            salt = self.name_l_edit.text().lower().encode('utf-8')
            passwd_hash = hashlib.pbkdf2_hmac(
                'sha512', passwd_bytes, salt, 10000)
            self.database.add_user(
                self.name_l_edit.text(),
                binascii.hexlify(passwd_hash))
            self.messages.information(
                self, 'Успех', 'Пользователь успешно зарегистрирован.')
            # Рассылаем клиентам сообщение о необходимости обновить справичники
            self.server.service_update_lists()
            self.close()



import time
import traceback
from functools import wraps
from socket import socket

from log.client_log_config import *
from dynaconf import settings


class Log:
    """
    Декоратор логирования
    """
    __slots__ = 'params'

    def __init__(self, params):
        self.params = params

    def __call__(self, func):
        @wraps(func)
        def decorated(*args, **kwargs):
            result = func(*args, *kwargs)
            # сообщение в лог
            message = f'{time.asctime()} Вызван декоратор {Log.__name__} для {func.__name__}'
            if args:
                message += '\targs: {}'.format(args)
            if kwargs:
                message += '\tkwargs: {}'.format(kwargs)
            if result:
                message += '\tРезультат = {}'.format(result)

            message += f' из функции {traceback.format_stack()[0].strip().split()[-1]}'

            if self.params:
                # logger.debug(
                #     f'Была вызвана функция {func.__name__} c параметрами {args} , {kwargs}. ')
                self.params.info(message)
            return result

        return decorated


# @Log(logger)
# def simple_func(a=2, v=4):
#     print(f'Тестовая функция, {a, v}')
#     return a + v


# Функция проверки, что клиент авторизован на сервере
# Проверяет, что передаваемый объект сокета находится в списке клиентов. Если его там нет закрывает сокет
def login_required(func):
    """
    Функция проверки, что клиент авторизован на сервере.
    Проверяет, что передаваемый объект сокета находится в списке клиентов.
    Если его там нет закрывает сокет
    :param func:
    :return:
    """
    def checker(*args, **kwargs):
        # Если первый аргумент - экземпляр Server
        # А сокет в остальных аргументах
        # Импортить необходимо тут, иначе ошибка рекурсивного импорта.
        from server import Server
        if isinstance(args[0], Server):
            found = False
            for arg in args:
                if isinstance(arg, socket):
                    # Проверяем, что данный сокет есть в списке names класса MessageProcessor
                    for client in args[0].names:
                        if args[0].names[client] == arg:
                            found = True

            # Теперь надо проверить, что передаваемые аргументы не presence сообщение
            for arg in args:
                if isinstance(arg, dict):
                    if settings.ACTION in arg and arg[settings.ACTION] == settings.PRESENCE:
                        found = True
            # Если не не авторизован и не сообщение начала авторизации, то вызываем исключение.
            if not found:
                raise TypeError

        return func(*args, **kwargs)

    return checker


def main():
    # Очистим все обработчики событий
    logger.handlers = []

    # Создадим обработчит только для этого модуля
    fh = logging.FileHandler('log_storage/app.main.log', encoding='utf-8')
    fh.setLevel(logging.INFO)
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    print("Запущен внутренний модуль логирования")
    # simple_func()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Date: 04.12.2019
# Author: MaximRaduntsev

import datetime
from server_db.models.server_models import User, UserHistory, \
    ActiveUser, LoginHistory, Contact
from server_db.data import session_factory

from pathlib import Path
from dynaconf import settings


class ServerStorage:
    """
    Класс - серверная база данных
    """

    def __init__(self):
        # Создаём движок базы данных
        db_def_name = Path(settings.get('DATABASES.SERVER.NAME'))
        session_factory.global_init(db_def_name)
        session_factory.create_tables()
        self.session = session_factory.create_session()

        # Если в таблице активных пользователей есть записи, то их необходимо удалить
        # Когда устанавливаем соединение, очищаем таблицу активных
        # пользователей
        if self.session.query(ActiveUser).count() > 0:
            self.session.query(ActiveUser).delete()
            self.session.commit()

    def get_user_by_name(self, username):
        """Возвращает объект пользователя по его имени."""
        user = self.session.query(User).filter(User.name == username.lower())
        return user.first() if user.count() else None

    def get_id_by_name(self, username):
        """
        Check client_name exist in clients table
        :param username:
        :return: record id if exist, -1 otherwise
        """
        user = self.session.query(User).filter(User.name == username).first()
        if user:
            return user.id
        else:
            exit(1)

    def get_user_by_like(self, query):
        """Возвращает объект пользователя по его имени."""
        users = self.session.query(
            User.name,
            User.last_login,
            UserHistory.sent,
            UserHistory.accepted).filter(
            User.id == UserHistory.id,
            User.name.ilike(f'%{query}%'))
        return users.all()

    def user_modify(self, username, new_username):
        """
        Функция изменения данных пользователя
        :param username:
        :param new_username:
        :return:
        """
        self.session.query(User).filter(
            User.name == username).update({'name': new_username})
        self.session.commit()

    def user_login(self, username, ip_address, port, key):
        """
        Функция выполняющяяся при входе пользователя, записывает в базу факт входа
        :param username:
        :param ip_address:
        :param port:
        :param key:
        :return:
        """
        # print(username, ip_address, port)
        # Запрос в таблицу пользователей на наличие там пользователя с таким именем
        # client = self.find_by_name(username)
        user = self.get_user_by_name(username)
        # Если имя пользователя уже присутствует в таблице, обновляем время
        # последнего входа
        if user:
            user.last_login = datetime.datetime.now()
            if user.pubkey != key:
                user.pubkey = key
        # Если нет, то генерируем исключение
        else:
            raise ValueError('Пользователь не зарегистрирован.')

        # Теперь можно создать запись в таблицу активных пользователей о факте
        # входа.
        new_active_user = ActiveUser(
            user.id, ip_address, port, datetime.datetime.now())
        self.session.add(new_active_user)

        # и сохранить в историю входов
        history = LoginHistory(
            user.id,
            datetime.datetime.now(),
            ip_address,
            port)
        self.session.add(history)

        # Сохрраняем изменения
        self.session.commit()

    def add_user(self, name, pass_hash):
        """
        Функция регистрации пользователя. Принимает имя и хэш пароля,
        создаёт запись в таблице статистики.
        :param name:
        :param pass_hash:
        :return:
        """
        user_row = User(name, pass_hash)
        self.session.add(user_row)
        self.session.commit()
        history_row = UserHistory(user_row.id)
        self.session.add(history_row)
        self.session.commit()

    def remove_user(self, name):
        """
        Функция удаляющая пользователя из базы
        :param name:
        :return:
        """
        user = self.get_user_by_name(name)
        self.session.query(ActiveUser).filter_by(user=user.id).delete()
        self.session.query(LoginHistory).filter_by(name=user.id).delete()
        self.session.query(Contact).filter_by(user=user.id).delete()
        self.session.query(Contact).filter_by(contact=user.id).delete()
        self.session.query(UserHistory).filter_by(user=user.id).delete()
        self.session.query(User).filter_by(name=name).delete()
        self.session.commit()

    def get_hash(self, name):
        """
        Функция возвращает хэш требуемго пользователя.
        :param name:
        :return:
        """
        user = self.get_user_by_name(name)
        return user.pass_hash

    def get_pubkey(self, name):
        """
        Функция возвращает публичный ключ пользователя
        :param name:
        :return:
        """
        user = self.get_user_by_name(name)
        return user.pubkey

    def check_user(self, name):
        """
        Функция проверки пользователя
        :param name:
        :return:
        """
        user = self.get_user_by_name(name)
        if user:
            return True
        else:
            return False

    def user_logout(self, username):
        """
        Функция фиксирующая отключение пользователя
        :param username:
        :return:
        """
        # Запрашиваем пользователя, что покидает нас
        # получаем запись из таблицы AllUsers
        user = self.get_user_by_name(username)
        # Удаляем его из таблицы активных пользователей.
        # Удаляем запись из таблицы ActiveUsers
        self.session.query(ActiveUser).filter_by(user=user.id).delete()

        # Применяем изменения
        self.session.commit()

    def process_message(self, sender, recipient):
        """
        Функция фиксирует передачу сообщения и делает соответствующие отметки в БД
        :param sender:
        :param recipient:
        :return:
        """
        # Получаем ID отправителя и получателя
        sender = self.get_id_by_name(sender)
        recipient = self.get_id_by_name(recipient)

        sender_row = self.session.query(
            UserHistory).filter_by(user=sender).first()
        sender_row.sent += 1
        recipient_row = self.session.query(
            UserHistory).filter_by(user=recipient).first()
        recipient_row.accepted += 1
        self.session.commit()

    def add_contact(self, user, contact):
        """
        Функция добавляет контакт для пользователя.
        :param user:
        :param contact:
        :return:
        """
        # Получаем ID пользователей
        user = self.get_user_by_name(user)
        contact = self.get_user_by_name(contact)

        # Проверяем что не дубль и что контакт может существовать (полю
        # пользователь мы доверяем)
        if not contact or self.session.query(Contact).filter_by(
                user=user.id, contact=contact.id).count():
            return

        # Создаём объект и заносим его в базу
        contact_row = Contact(user.id, contact.id)
        self.session.add(contact_row)
        self.session.commit()

    def remove_contact(self, user, contact):
        """
        Функция удаляет контакт из базы данных
        :param user:
        :param contact:
        :return:
        """
        # Получаем ID пользователей
        user = self.get_user_by_name(user)
        contact = self.get_user_by_name(contact)

        # Проверяем что контакт может существовать (полю пользователь мы
        # доверяем)
        if not contact:
            return

        # Удаляем требуемое
        self.session.query(Contact).filter(
            Contact.user == user.id,
            Contact.contact == contact.id
        ).delete()
        self.session.commit()

    def users_list(self):
        """
        Функция возвращает список известных пользователей со временем последнего входа.
        :return:
        """
        query = self.session.query(
            User.name,
            User.last_login,
        )
        # Возвращаем список кортежей
        return query.all()

    # Функция возвращает список активных пользователей
    def active_users_list(self):
        """
        Функция возвращает список активных пользователей
        Запрашиваем соединение таблиц и собираем кортежи имя, адрес, порт, время.
        :return:
        """
        query = self.session.query(
            User.id,
            User.name,
            ActiveUser.ip_address,
            ActiveUser.port,
            ActiveUser.login_time
        ).join(User)
        # Возвращаем список кортежей
        return query.all()

    def login_history(self, username=None):
        """
        Функция возвращающая историю входов по пользователю или всем пользователям
        :param username:
        :return:
        """
        # Запрашиваем историю входа
        query = self.session.query(User.name,
                                   LoginHistory.ip,
                                   LoginHistory.port,
                                   LoginHistory.date_time
                                   ).join(User)
        # Если было указано имя пользователя, то фильтруем по нему
        if username:
            query = query.filter(User.name == username)
        return query.all()

    def get_contacts(self, username):
        """
        Функция возвращает список контактов пользователя.
        :param username:
        :return:
        """
        # Запрашивааем указанного пользователя
        user = self.session.query(User).filter_by(name=username).one()

        # Запрашиваем его список контактов
        query = self.session.query(Contact, User.name). \
            filter_by(user=user.id). \
            join(User, Contact.contact == User.id)

        # выбираем только имена пользователей и возвращаем их.
        return [contact[1] for contact in query.all()]

    def message_history(self):
        """
        Функция возвращает количество переданных и полученных сообщений
        :return:
        """
        query = self.session.query(
            User.name,
            User.last_login,
            UserHistory.sent,
            UserHistory.accepted
        ).join(User)
        # Возвращаем список кортежей
        return query.all()


# Отладка
if __name__ == '__main__':
    test_db = ServerStorage()
    # test_db.user_login('1111', '192.168.1.113', 8080)
    # test_db.user_login('McG', '192.168.1.113', 8081)
    # print(test_db.users_list())
    # print(test_db.active_users_list())
    # # test_db.user_logout('McG')
    # # print(test_db.login_history('re'))
    # # test_db.add_contact('test2', 'test1')
    # # test_db.add_contact('test1', 'test3')
    # # test_db.add_contact('test1', 'test6')
    # # test_db.remove_contact('test1', 'test3')
    # test_db.process_message('McG', '1111')
    # print(test_db.message_history())

    # print(test_db.get_user_by_like('dd'))
    # print(test_db.get_user_by_name('ddd'))

    test_db.user_modify('yuyuy', 'new2020')



from setuptools import setup, find_packages

setup(
    name="pyside2_mess_server",
    version="0.1",
    description="PySide2 messenger - Server package",
    author="MaximR",
    author_email="support@geekbrains.ru",
    packages=find_packages(),
    install_requires=['dynaconf', 'sqlalchemy', 'pycryptodome', 'pyside2']
)



# -*- coding: utf-8 -*-
# Date: 04.12.2019
# Author: MaximRaduntsev

import os


def get_full_path(db_filename: str) -> str:
    this_folder = os.path.dirname(__file__)
    return os.path.join(this_folder, db_filename)



# Date: 20/09/2019 01:23
# Author: MaximRaduntsev
import binascii
import hmac
import json
import os
import sys
import threading
import time
from socket import socket, AF_INET, SOCK_STREAM
import select

from common.arg_parser import create_parser
from common.descriptors import AddressField
from common.metaclasses import ServerVerifier
from common.utils import send_message, get_message
from server_db.server_data_services import ServerStorage
from jim.core import Message as Obj

from dynaconf import settings

from common.decos import Log, login_required
from log.server_log_config import logger

from PySide2.QtWidgets import QApplication, QTableWidgetItem
from PySide2.QtCore import Qt, QTimer
from ui.server_gui import Main

log = Log(logger)
# Флаг что был подключён новый пользователь, нужен чтобы не мучать BD постоянными
# запросами на обновление
new_connection = False
conflag_lock = threading.Lock()


class Server(threading.Thread, metaclass=ServerVerifier):
    address = AddressField()
    """
    Основной класс сервера. Принимает содинения, словари - пакеты
    от клиентов, обрабатывает поступающие сообщения.
    Работает в качестве отдельного потока.
    """
    def __init__(self, address: (str, int), database):
        super().__init__()
        # База данных сервера
        self.database = database

        self.sock = None
        self.address = address
        # Список подключённых клиентов.
        self.clients = []
        # Список сообщений на отправку.
        self.messages = []

        # Сокеты
        self.listen_sockets = None
        self.error_sockets = None

        # Флаг продолжения работы
        self.running = True

        # Словарь содержащий сопоставленные имена и соответствующие им сокеты.
        self.names = dict()

    def connected(self):
        """
        Функция открывает сокет сервера
        """
        transport = socket(AF_INET, SOCK_STREAM)
        transport.bind(self.address)
        transport.settimeout(0.5)

        # Начинаем слушать сокет.
        self.sock = transport
        self.sock.listen(settings.get('MAX_CONNECTIONS'))

        info_msg = f'Запущен сервер ip: {self.address[0]}, port: {self.address[1]}. ' \
                   f'Если адрес не указан, принимаются соединения с любых адресов'
        logger.info(info_msg)

    def process_message(self, request):
        """
        Функция адресной отправки сообщения определённому клиенту. Принимает словарь сообщение,
        список зарегистрированых пользователей и слушающие сокеты. Ничего не возвращает.
        """
        message = Obj(request)
        if message.destination in self.names and self.names[message.destination] \
                in self.listen_sockets:
            try:
                send_message(self.names[message.destination], message.to_dict)
                info_msg = f'Отправлено сообщение пользователю {message.destination} ' \
                           f'от пользователя {message.sender}'
                logger.info(info_msg)
            except OSError:
                self.remove_client(message.destination)
        elif message.destination in self.names and self.names[message.destination] not in \
                self.listen_sockets:
            logger.error(
                f'Связь с клиентом {message.destination} была потеряна. Соединение закрыто, '
                f'доставка невозможна.')
            self.remove_client(self.names[message.destination])
            # raise ConnectionError
        else:
            critical_msg = f'Пользователь {message.destination} не зарегистрирован ' \
                           f'на сервере, отправка сообщения невозможна'
            logger.critical(critical_msg)

    @login_required
    def run_action(self, request, client):
        """
        Обработчик сообщений от клиентов, принимает словарь - сообщение
        от клиента, проверяет корректность, отправляет словарь-ответ в
        случае необходимости.
        """
        message = Obj(request)
        logger.info(f'Разбор сообщения от клиента : {message.to_dict}')
        # Если это сообщение о присутствии, принимаем и отвечаем
        if settings.ACTION in message.to_dict and message.action == settings.PRESENCE and \
                settings.TIME in message.to_dict and settings.USER in message.to_dict:
            # Если такой пользователь ещё не зарегистрирован, регистрируем, иначе
            # отправляем ответ и завершаем соединение.
            self.authorize_user(message, client)

        # Если это сообщение, то добавляем его в очередь сообщений. Ответ не
        # требуется.
        elif settings.ACTION in message.to_dict and message.action == settings.MESSAGE and \
                settings.DESTINATION in message.to_dict and settings.TIME in message.to_dict and \
                settings.SENDER in message.to_dict and \
                settings.MESSAGE_TEXT in message.to_dict and \
                self.names[message.sender] == client:
            if message.destination in self.names:
                self.database.process_message(
                    message.sender, message.destination)
                self.process_message(message.to_dict)
                try:
                    send_message(client, Obj.success(settings.OK).to_dict)
                except OSError:
                    self.remove_client(client)
            else:
                response = 'Пользователь не зарегистрирован на сервере.'
                try:
                    send_message(client, Obj.error_resp(response).to_dict)
                except OSError:
                    pass
            return

        # Если это запрос контакт-листа
        elif settings.ACTION in message.to_dict and message.action == settings.GET_CONTACTS and \
                settings.USER in message.to_dict and self.names[message.user] == client:
            response = settings.RESPONSE_ACCEPTED
            response[settings.LIST_INFO] = self.database.get_contacts(
                message.user)
            send_message(client, response)

        # Если это добавление контакта
        elif settings.ACTION in message.to_dict and message.action == settings.ADD_CONTACT and \
                settings.ACCOUNT_NAME in message.to_dict and settings.USER in message.to_dict and \
                self.names[message.user] == client:
            self.database.add_contact(message.user,
                                      message.to_dict[settings.ACCOUNT_NAME])
            send_message(client, Obj.success(settings.OK).to_dict)

        # Если это удаление контакта
        elif settings.ACTION in message.to_dict and message.action == settings.DEL_CONTACT and \
                settings.ACCOUNT_NAME in message.to_dict and settings.USER in message.to_dict and \
                self.names[message.user] == client:
            self.database.remove_contact(
                message.user, message.to_dict[settings.ACCOUNT_NAME])
            send_message(client, Obj.success(settings.OK).to_dict)

        # Если это запрос известных пользователей
        elif settings.ACTION in message.to_dict and message.action == settings.USERS_REQUEST and \
                settings.ACCOUNT_NAME in message.to_dict and self.names[message.user] == client:
            response = settings.RESPONSE_ACCEPTED
            response[settings.LIST_INFO] = [user[0]
                                            for user in self.database.users_list()]
            send_message(client, response)

        # Если клиент выходит
        elif settings.ACTION in message.to_dict and message.action == settings.QUIT and \
                settings.ACCOUNT_NAME in message.to_dict and self.names[message.user] == client:
            self.remove_client(client)

            # Если это запрос публичного ключа пользователя
        elif settings.ACTION in message.to_dict and \
                message.action == settings.PUBLIC_KEY_REQUEST and \
                settings.ACCOUNT_NAME in message.to_dict:
            response = settings.RESPONSE_NET_AUTH_REQUIRED
            response[settings.DATA] = self.database.get_pubkey(message.user)
            # может быть, что ключа ещё нет (пользователь никогда не логинился,
            # тогда шлём 400)
            if response[settings.DATA]:
                try:
                    send_message(client, response)
                except OSError:
                    self.remove_client(client)
            else:
                error_msg = 'Нет публичного ключа для данного пользователя'
                try:
                    send_message(client, Obj.error_resp(error_msg).to_dict)
                except OSError:
                    self.remove_client(client)
        else:
            error_msg = 'Действие недоступно'
            try:
                send_message(client, Obj.error_resp(error_msg).to_dict)
                return logger.error(error_msg)
            except OSError:
                self.remove_client(client)

    def write_responses(self, w_clients):
        """
        Функция отправки ответа сервера клиентам, от которых были запросы
        """
        for sock in self.messages:
            try:
                self.process_message(sock, w_clients)
            except Exception as e:
                error_msg = f'Связь с клиентом с именем {sock.destination} была потеряна, {e}'
                logger.critical(error_msg)
                self.database.user_logout(sock.destination)
                self.clients.remove(self.names[sock.destination])
                del self.names[sock.destination]
        self.messages.clear()

    def run(self):
        """
        Запуск основного цикла
        :return:
        """
        # Готовим сокет
        self.connected()

        while self.running:
            try:
                # Ждём подключения, если таймаут вышел, ловим исключение.
                client, addr = self.sock.accept()
            except OSError:
                pass
            else:
                info_msg = f'Получен запрос на соединение от {addr}'
                logger.info(info_msg)
                self.clients.append(client)

            r = []
            w = []
            e = []
            # Проверяем на наличие ждущих клиентов
            try:
                if self.clients:
                    r, self.listen_sockets, self.error_sockets = \
                        select.select(self.clients, self.clients, [], 0)
            except OSError as err:
                logger.error(f'Ошибка работы с сокетами: {err.errno}')
            # принимаем сообщения и если ошибка, исключаем клиента.
            if r:
                for sock in r:
                    try:
                        self.run_action(get_message(sock), sock)
                    except (OSError, json.JSONDecodeError, TypeError):
                        self.remove_client(sock)

    def remove_client(self, client):
        """
        Функция обработчик клиента с которым потеряна связь
        ищет клиента в словаре клиентов и удаляет его со списков и базы:
        :param client:
        :return:
        """
        logger.info(f'Клиент {client.getpeername()} отключился от сервера.')
        for name in self.names:
            if self.names[name] == client:
                self.database.user_logout(name)
                del self.names[name]
                break
        self.clients.remove(client)
        client.close()
        logger.debug('closed')

    def authorize_user(self, message: Obj, sock):
        """
        Функция авторизации пользователя на сервере
        :param message:
        :param sock:
        :return:
        """
        # Если имя пользователя уже занято то возвращаем 400
        if message.user in self.names.keys():
            error_msg = 'Имя пользователя уже занято'
            try:
                send_message(sock, Obj.error_resp(error_msg).to_dict)
            except OSError:
                pass
            self.clients.remove(sock)
            sock.close()
        # Проверяем что пользователь зарегистрирован на сервере.
        elif not self.database.check_user(message.user):
            error_msg = 'Пользователь не зарегистрирован'
            try:
                send_message(sock, Obj.error_resp(error_msg).to_dict)
            except OSError:
                pass
            self.clients.remove(sock)
            sock.close()
        else:
            # Иначе отвечаем 501 и проводим процедуру авторизации
            # Словарь - заготовка
            message_auth = settings.RESPONSE_NET_AUTH_REQUIRED
            # Набор байтов в hex представлении
            random_str = binascii.hexlify(os.urandom(64))
            # В словарь байты нельзя, декодируем (json.dumps -> TypeError)
            message_auth[settings.DATA] = random_str.decode('ascii')
            # Создаём хэш пароля и связки с рандомной строкой, сохраняем
            # серверную версию ключа
            hash = hmac.new(self.database.get_hash(message.user), random_str)
            digest = hash.digest()
            try:
                # Обмен с клиентом
                send_message(sock, message_auth)
                ans = get_message(sock)
            except OSError:
                sock.close()
                return
            client_digest = binascii.a2b_base64(ans[settings.DATA])
            # Если ответ клиента корректный, то сохраняем его в список
            # пользователей.
            if settings.RESPONSE in ans and ans[settings.RESPONSE] == \
                    settings.NETWORK_AUTHENTICATION_REQUIRED and \
                    hmac.compare_digest(digest, client_digest):
                self.names[message.user] = sock
                client_ip, client_port = sock.getpeername()
                try:
                    send_message(sock, Obj.success(settings.OK).to_dict)
                except OSError:
                    self.remove_client(message.user)
                # добавляем пользователя в список активных и если у него изменился открытый ключ
                # сохраняем новый
                self.database.user_login(message.user,
                                         client_ip, client_port,
                                         message.public_key)
            else:
                error_msg = 'Неверный пароль'
                try:
                    send_message(sock, Obj.error_resp(error_msg).to_dict)
                except OSError:
                    pass
                self.clients.remove(sock)
                sock.close()

    def service_update_lists(self):
        """
        Функция - отправляет сервисное сообщение 205 с требованием
        клиентам обновить списки
        :return:
        """
        for client in self.names:
            try:
                send_message(
                    self.names[client],
                    settings.RESPONSE_RESET_CONTENT)
            except OSError:
                self.remove_client(self.names[client])


def main():
    # Загрузка параметров командной строки, если нет параметров, то задаём
    # значения по умолчанию.
    parser = create_parser(False)
    gui_flag = False
    # Инициализация базы данных
    db = ServerStorage()

    address = (parser.parse_args().ip, parser.parse_args().port)
    # gui_flag = parser.parse_args().no_gui

    server = Server(address, db)
    server.daemon = True
    server.start()
    time.sleep(0.1)

    # Если  указан параметр без GUI то запускаем простенький обработчик
    # консольного ввода
    if gui_flag:
        while True:
            command = input('Введите exit для завершения работы сервера.')
            if command == 'exit':
                # Если выход, то завршаем основной цикл сервера.
                server.running = False
                server.join()
                break

    # Если не указан запуск без GUI, то запускаем GUI:
    else:
        # Создаём графическое окуружение для сервера:
        server_app = QApplication(sys.argv)
        main_window = Main(db, server)

        def display_table(f, table):
            # table.setFont(QFont("Helvetica", 10))
            row = f()
            table.setRowCount(len(row))
            column_count = table.columnCount()

            for i, data in enumerate(row):
                for j in range(column_count):
                    item = QTableWidgetItem()
                    item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
                    item.setText(str(data[j]))
                    table.setItem(i, j, item)

        display_table(db.message_history, main_window.history_user_table)

        # Функция обновляющяя список подключённых, проверяет флаг подключения,
        # и если надо обновляет список
        def list_update():
            display_table(db.active_users_list, main_window.active_user_table)

        timer = QTimer()
        timer.timeout.connect(list_update)
        timer.start(1000)  # trigger every 1 s.

        # Запускаем GUI
        server_app.exec_()


if __name__ == '__main__':
    main()



# Date: 20/09/2019 01:15
# Author: MaximRaduntsev

from dynaconf import settings
from datetime import datetime
import json

from common.decos import Log
from log.client_log_config import logger
log = Log(logger)


def write_bytes(dict_msg):
    """
    Функция принимает словарь, преобразует его в JSON-объект и кодирует объект в байты
    :param dict_msg: dict (Словарь с данными)
    :return: bytes (Байтовое представление JSON-объекта)
    """
    if isinstance(dict_msg, dict):
        byte_message = json.dumps(dict_msg).encode(settings.BASE_ENCODING)
        return byte_message
    raise TypeError


def read_bytes(byte_msg):
    """
    Функция декодировщик байтов в словарь
    :param byte_msg: bytes (Байтовое представление JSON-объекта)
    :return: dict (Словарь с данными)
    """
    if isinstance(byte_msg, bytes):
        dict_message = json.loads(byte_msg.decode(settings.BASE_ENCODING))
        if isinstance(dict_message, dict):
            return dict_message
        raise TypeError
    raise TypeError


@log
def send_message(sock, message):
    """
    Функция отправки сообщения
    :param sock: socket (Объект сокета)
    :param message: dict (Словарь сообщения)
    :return: None
    """
    sock.send(write_bytes(message))


@log
def get_message(sock):
    """
    Функция приема сообщения
    :param sock: socket (Объект сокета)
    :return: dict (Словарь сообщения)
    """
    return read_bytes(sock.recv(settings.MAX_PACKAGE_LENGTH))


def get_time():
    """
    Функция возвращает текущее время
    :return: str (HH:MM:SS)
    """
    return datetime.now().strftime("%H:%M:%S")



# Date: 02.12.2019 20:36
# Author: MaximRaduntsev

import argparse
from dynaconf import settings


def create_parser(client=True):
    """
    Функция парсинга ключей. Парсер аргументов командной строки, возвращает кортеж из
    4 элементов: адрес сервера, порт, имя пользователя, пароль.
    Выполняет проверку на корректность номера порта.
    :return: список ключей
    """
    ip = settings.DEFAULT_CLIENT_IP if client else settings.DEFAULT_SERVER_IP

    parser = argparse.ArgumentParser()

    parser.add_argument(
        'ip',
        default=ip,
        type=str,
        nargs='?',
        help=f'Задать IP адресс, по умолчанию {ip}')
    parser.add_argument(
        'port',
        default=settings.DEFAULT_PORT,
        type=int,
        nargs='?',
        help=f'Установить PORT, по умолчанию {settings.DEFAULT_PORT}')
    parser.add_argument('name', default=None, type=str, nargs='?')
    parser.add_argument('password', default='', nargs='?')
    parser.add_argument('no_gui', action='store_true')

    return parser
